@@ -4,7 +4,7 @@ import datetime
 import logging
 from collections import namedtuple  # pylint: disable=syntax-error
 
-from .classes import _AttributeList, _AttributeTagsList, _AttributeFinderInfo
+from .classes import _AttributeFinderInfo, _AttributeList, _AttributeTagsList
 from .constants import *
 from .datetime_utils import (
     datetime_has_tz,
@@ -286,10 +286,6 @@ def validate_attribute_value(attribute, value):
         value may be a single value or a list depending on what attribute expects 
         if value contains None, returns None """
 
-    logging.debug(
-        f"validate_attribute_value: attribute: {attribute}, value: {value}, type: {type(value)}"
-    )
-
     # check to see if we got None
     try:
         if None in value:
@@ -358,8 +354,6 @@ def validate_attribute_value(attribute, value):
             raise TypeError(f"Unknown type: {type(val)}")
         new_values.append(new_val)
 
-    logging.debug(f"new_values = {new_values}")
-
     if attribute.list:
         return new_values
     else:
