@@ -5,13 +5,16 @@
 import datetime
 import json
 import logging
+import os.path
 import pathlib
 import plistlib
 import sys
+import base64
 
 # plistlib creates constants at runtime which causes pylint to complain
 from plistlib import FMT_BINARY  # pylint: disable=E0611
 
+import applescript
 import xattr
 
 from ._version import __version__
@@ -45,7 +48,6 @@ from .datetime_utils import (
     datetime_utc_to_local,
 )
 from .debug import _debug, _get_logger, _set_debug
-from .findercomments import clear_finder_comment, set_finder_comment
 from .findertags import Tag, get_tag_color_name, set_finderinfo_color
 
 __all__ = [
@@ -71,6 +73,27 @@ __all__ = [
 # TODO: check what happens if OSXMetaData.__init__ called with invalid file--should result in error but saw one case where it didn't
 # TODO: cleartags does not always clear colors--this is a new behavior, did Mac OS change something in implementation of colors?
 
+# AppleScript for manipulating Finder comments
+_scpt_set_finder_comment = applescript.AppleScript(
+    """
+            on run {path, fc}
+	            set thePath to path
+	            set theComment to fc
+	            tell application "Finder" to set comment of (POSIX file thePath as alias) to theComment
+            end run
+            """
+)
+
+_scpt_clear_finder_comment = applescript.AppleScript(
+    """
+            on run {path}
+	            set thePath to path
+	            set theComment to missing value
+	            tell application "Finder" to set comment of (POSIX file thePath as alias) to theComment
+            end run
+            """
+)
+
 
 class OSXMetaData:
     """Create an OSXMetaData object to access file metadata"""
@@ -97,10 +120,10 @@ class OSXMetaData:
 
     def __init__(self, fname, tz_aware=False):
         """Create an OSXMetaData object to access file metadata
-            fname: filename to operate on
-            timezone_aware: bool; if True, date/time attributes will return 
-                      timezone aware datetime.dateime attributes; if False (default)
-                      date/time attributes will return timezone naive objects """
+        fname: filename to operate on
+        timezone_aware: bool; if True, date/time attributes will return
+                  timezone aware datetime.dateime attributes; if False (default)
+                  date/time attributes will return timezone naive objects"""
         self._fname = pathlib.Path(fname)
         self._posix_name = self._fname.resolve().as_posix()
         self._tz_aware = tz_aware
@@ -125,24 +148,32 @@ class OSXMetaData:
 
     @property
     def name(self):
-        """ POSIX path of the file OSXMetaData is operating on """
+        """POSIX path of the file OSXMetaData is operating on"""
         return self._fname.resolve().as_posix()
 
     @property
     def tz_aware(self):
-        """ returns the timezone aware flag """
+        """returns the timezone aware flag"""
         return self._tz_aware
 
     @tz_aware.setter
     def tz_aware(self, tz_flag):
-        """ sets the timezone aware flag
-            tz_flag: bool """
+        """sets the timezone aware flag
+        tz_flag: bool"""
         self._tz_aware = tz_flag
 
-    def asdict(self):
-        """ Return dict with all attributes for this file """
+    def asdict(self, all_=False, encode=True):
+        """Return dict with all attributes for this file
 
-        attribute_list = self.list_metadata()
+        Args:
+            all_: bool, if True, returns all attributes including those that osxmetadata knows nothing about
+            encode: bool, if True, encodes values for unknown attributes with base64, otherwise leaves the values as raw bytes
+
+        Returns:
+            dict with attributes for this file
+        """
+
+        attribute_list = self._list_attributes() if all_ else self.list_metadata()
         dict_data = {
             "_version": __version__,
             "_filepath": self._posix_name,
@@ -172,20 +203,37 @@ class OSXMetaData:
                     # get raw value
                     dict_data[attribute.constant] = self.get_attribute(attribute.name)
             except KeyError:
-                # unknown attribute, ignore it
-                pass
+                # an attribute osxmetadata doesn't know about
+                if all_:
+                    try:
+                        value = self._attrs[attr]
+                        # convert value to base64 encoded ascii
+                        if encode:
+                            value = base64.b64encode(value).decode("ascii")
+                        dict_data[attr] = value
+                    except KeyError as e:
+                        # value disappeared between call to _list_attributes and now
+                        pass
         return dict_data
 
-    def to_json(self):
-        """ Returns a string in JSON format for all attributes in this file """
-        dict_data = self.asdict()
+    def to_json(self, all_=False):
+        """Returns a string in JSON format for all attributes in this file
+    
+        Args:
+            all_: bool; if True, also restores attributes not known to osxmetadata (generated with asdict(all_=True, encode=True) )
+        """
+        dict_data = self.asdict(all_=all_)
         return json.dumps(dict_data)
 
-    def _restore_attributes(self, attr_dict):
-        """ restore attributes from attr_dict
-            for each attribute in attr_dict, will set the attribute
-            will not clear/erase any attributes on file that are not in attr_dict
-            attr_dict: an attribute dict as produced by OSXMetaData.asdict() """
+    def _restore_attributes(self, attr_dict, all_=False):
+        """restore attributes from attr_dict
+        for each attribute in attr_dict, will set the attribute
+        will not clear/erase any attributes on file that are not in attr_dict
+
+        Args:
+            attr_dict: an attribute dict as produced by OSXMetaData.asdict()
+            all_: bool; if True, also restores attributes not known to osxmetadata (generated with asdict(all_=True, encode=True) )
+        """
 
         for key, val in attr_dict.items():
             if key.startswith("_"):
@@ -204,20 +252,21 @@ class OSXMetaData:
                             f"expected list for attribute {key} but got {type(val)}"
                         )
                     self.set_attribute(key, Tag(val[0], val[1]))
-                else:
+                elif key in ATTRIBUTES:
                     self.set_attribute(key, val)
+                elif all_:
+                    self._attrs.set(key, base64.b64decode(val))
             except Exception as e:
                 logging.warning(
                     f"Unable to restore attribute {key} for {self._fname}: {e}"
                 )
 
     def get_attribute(self, attribute_name):
-        """ load attribute and return value or None if attribute was not set 
-            attribute_name: name of attribute
+        """load attribute and return value or None if attribute was not set
+        attribute_name: name of attribute
         """
 
         attribute = ATTRIBUTES[attribute_name]
-        logging.debug(f"get: {attribute}")
 
         # user tags and finderinfo need special processing
         if attribute.name == "tags":
@@ -257,10 +306,10 @@ class OSXMetaData:
             return plist
 
     def get_attribute_str(self, attribute_name):
-        """ returns a string representation of attribute value
-            e.g. if attribute is a datedate.datetime object, will 
-            format using datetime.isoformat()
-            attribute_name: name of attribute """
+        """returns a string representation of attribute value
+        e.g. if attribute is a datedate.datetime object, will
+        format using datetime.isoformat()
+        attribute_name: name of attribute"""
         value = self.get_attribute(attribute_name)
         if type(value) in [list, set]:
             if value:
@@ -288,13 +337,11 @@ class OSXMetaData:
             return str(value)
 
     def set_attribute(self, attribute_name, value):
-        """ write attribute to file with value
-            attribute_name: an osxmetadata Attribute name
-            value: value to store in attribute """
+        """write attribute to file with value
+        attribute_name: an osxmetadata Attribute name
+        value: value to store in attribute"""
         attribute = ATTRIBUTES[attribute_name]
 
-        logging.debug(f"set_attribute: {attribute_name} {attribute} {value}")
-
         # user tags need special processing to normalize names
         if attribute.name == "tags":
             return self.tags.set_value(value)
@@ -308,7 +355,7 @@ class OSXMetaData:
         if attribute.name in _FINDER_COMMENT_NAMES:
             # Finder Comment needs special handling
             # code following will also set the attribute for Finder Comment
-            set_finder_comment(self._posix_name, value)
+            self.set_finder_comment(self._posix_name, value)
         elif attribute.class_ in [_AttributeList, _AttributeTagsList]:
             getattr(self, attribute.name).set_value(value)
         else:
@@ -319,48 +366,49 @@ class OSXMetaData:
         return value
 
     def update_attribute(self, attribute_name, value):
-        """ Update attribute with union of itself and value
-            (this avoids adding duplicate values to attribute)
-            attribute: an osxmetadata Attribute name
-            value: value to append to attribute """
+        """Update attribute with union of itself and value
+        (this avoids adding duplicate values to attribute)
+        attribute: an osxmetadata Attribute name
+        value: value to append to attribute"""
         return self.append_attribute(attribute_name, value, update=True)
 
     def append_attribute(self, attribute_name, value, update=False):
-        """ append value to attribute
-            attribute_name: an osxmetadata Attribute name
-            value: value to append to attribute
-            update: (bool) if True, update instead of append (e.g. avoid adding duplicates)
-                    (default is False) """
+        """append value to attribute
+        attribute_name: an osxmetadata Attribute name
+        value: value to append to attribute
+        update: (bool) if True, update instead of append (e.g. avoid adding duplicates)
+                (default is False)"""
 
         attribute = ATTRIBUTES[attribute_name]
-        logging.debug(f"append_attribute: {attribute} {value}")
 
         # start with existing values
         new_value = self.get_attribute(attribute.name)
 
         # user tags need special processing to normalize names
         if attribute.name == "tags":
-            if isinstance(new_value, list) and isinstance(value, list):
-                if update:
-                    # verify not already in values
-                    for val in value:
-                        if val not in new_value:
-                            new_value.append(val)
-                else:
-                    new_value.extend(value)
-                return self.tags.set_value(new_value)
-            else:
+            if not isinstance(new_value, list) or not isinstance(value, list):
                 raise TypeError(
                     f"tags expects values in list: {type(new_value)}, {type(value)}"
                 )
 
+            if update:
+                # verify not already in values
+                for val in value:
+                    if val not in new_value:
+                        new_value.append(val)
+            else:
+                new_value.extend(value)
+            return self.tags.set_value(new_value)
         if attribute.name == "finderinfo":
             raise ValueError(f"cannot append or update finderinfo")
 
         value = validate_attribute_value(attribute, value)
 
         if attribute.list:
-            if new_value is not None:
+            if new_value is None:
+                # no original value
+                new_value = value
+            else:
                 new_value = list(new_value)
                 if update:
                     for val in value:
@@ -368,25 +416,19 @@ class OSXMetaData:
                             new_value.append(val)
                 else:
                     new_value.extend(value)
-            else:
-                # no original value
-                new_value = value
         else:
             # scalar value
             if update:
                 raise AttributeError(f"Cannot use update on {attribute.type_}")
-            if new_value is not None:
-                new_value += value
-            else:
+            if new_value is None:
                 new_value = value
 
+            else:
+                new_value += value
         try:
             if attribute.name in _FINDER_COMMENT_NAMES:
                 # Finder Comment needs special handling
-                # code following will also set the attribute for Finder Comment
-                set_finder_comment(self._posix_name, new_value)
-                plist = plistlib.dumps(new_value, fmt=FMT_BINARY)
-                self._attrs.set(attribute.constant, plist)
+                self.set_finder_comment(self._posix_name, new_value)
             elif attribute.class_ in [_AttributeList, _AttributeTagsList]:
                 # if tags, set_value will normalize
                 getattr(self, attribute.name).set_value(new_value)
@@ -400,9 +442,9 @@ class OSXMetaData:
         return new_value
 
     def remove_attribute(self, attribute_name, value):
-        """ remove a value from attribute, raise ValueError if attribute does not contain value
-            only applies to multi-valued attributes, otherwise raises TypeError
-            attribute_name: name of OSXMetaData attribute """
+        """remove a value from attribute, raise ValueError if attribute does not contain value
+        only applies to multi-valued attributes, otherwise raises TypeError
+        attribute_name: name of OSXMetaData attribute"""
 
         attribute = ATTRIBUTES[attribute_name]
 
@@ -419,10 +461,10 @@ class OSXMetaData:
             self.set_attribute(attribute.name, values)
 
     def discard_attribute(self, attribute_name, value):
-        """ remove a value from attribute, unlike remove, does not raise exception
-            if attribute does not contain value
-            only applies to multi-valued attributes, otherwise raises TypeError
-            attribute_name: name of OSXMetaData attribute """
+        """remove a value from attribute, unlike remove, does not raise exception
+        if attribute does not contain value
+        only applies to multi-valued attributes, otherwise raises TypeError
+        attribute_name: name of OSXMetaData attribute"""
 
         attribute = ATTRIBUTES[attribute_name]
 
@@ -437,8 +479,8 @@ class OSXMetaData:
             pass
 
     def clear_attribute(self, attribute_name):
-        """ clear anttribute (remove it from the file) 
-            attribute_name: name of OSXMetaData attribute """
+        """clear anttribute (remove it from the file)
+        attribute_name: name of OSXMetaData attribute"""
 
         attribute = ATTRIBUTES[attribute_name]
 
@@ -446,11 +488,10 @@ class OSXMetaData:
             if attribute.name in _FINDER_COMMENT_NAMES:
                 # Finder Comment needs special handling
                 # code following will also clear the attribute for Finder Comment
-                clear_finder_comment(self._posix_name)
+                self.clear_finder_comment(self._posix_name)
 
             if attribute.name in ["finderinfo", "tags"]:
                 # don't clear the entire FinderInfo attribute, just delete the color
-                logging.debug(f"clearing finderinfo color")
                 set_finderinfo_color(str(self._fname), FINDER_COLOR_NONE)
 
             if attribute.name != "finderinfo":
@@ -461,14 +502,14 @@ class OSXMetaData:
             pass
 
     def _list_attributes(self):
-        """ list the attributes set on the file """
+        """list all the attributes set on the file"""
         return self._attrs.list()
 
     def list_metadata(self):
-        """ list the Apple metadata attributes set on the file:
-            e.g. those in com.apple.metadata namespace """
+        """list the Apple metadata attributes set on the file:
+        e.g. those in com.apple.metadata namespace"""
         # also lists com.osxmetadata.test used for debugging
-        mdlist = self._attrs.list()
+        mdlist = self._list_attributes()
         mdlist = [
             md
             for md in mdlist
@@ -478,18 +519,46 @@ class OSXMetaData:
         ]
         return mdlist
 
+    def set_finder_comment(self, path, comment):
+        """set finder comment for object path (file or directory)
+        path: path to file or directory in posix format
+        comment: comment string
+        """
+        if not os.path.exists(path):
+            raise FileNotFoundError(f"Could not find {path}")
+
+        if comment:
+            _scpt_set_finder_comment.run(path, comment)
+            plist = plistlib.dumps(comment, fmt=FMT_BINARY)
+            self._attrs.set(ATTRIBUTES["findercomment"].constant, plist)
+        else:
+            self.clear_finder_comment(path)
+
+    def clear_finder_comment(self, path):
+        """clear finder comment for object path (file or directory)
+        path: path to file or directory in posix format
+        """
+        if not os.path.exists(path):
+            raise FileNotFoundError(f"Could not find {path}")
+
+        _scpt_clear_finder_comment.run(path)
+        try:
+            self._attrs.remove(ATTRIBUTES["findercomment"].constant)
+        except (IOError, OSError):
+            # exception raised if attribute not found and attempt to remove it
+            pass
+
     def __getattr__(self, name):
-        """ if attribute name is in ATTRIBUTE dict, return the value
-            otherwise raise KeyError """
+        """if attribute name is in ATTRIBUTE dict, return the value
+        otherwise raise KeyError"""
         return self.get_attribute(name)
 
     def __setattr__(self, name, value):
-        """ if object is initialized and name is an attribute in ATTRIBUTES, 
-            set the attribute to value
-            if value value is None, will clear (delete) the attribute and all associated values
-            if name is not a metadata attribute, assume it's a normal class attribute and pass to
-            super() to handle  """
-        logging.debug(f"__setattr__: {name} {value}")
+        """if object is initialized and name is an attribute in ATTRIBUTES,
+        set the attribute to value
+        if value value is None, will clear (delete) the attribute and all associated values
+        if name is not a metadata attribute, assume it's a normal class attribute and pass to
+        super() to handle"""
         try:
             if self.__init:
                 # already initialized
